/* 
 * <START_COPYRIGHT>
 * Copyright (c) 2005 University of Southampton
 * See CPL.txt for license information.
 * http://www.opensource.org/license/cpl1.0.txt
 * <END_COPYRIGHT>
 */

package org.gridprovenance.client;

import org.w3c.dom.Element;
import java.security.Principal;
import java.net.URI;
import org.apache.axis.message.addressing.EndpointReference;
import org.gridprovenance.client.documentationstyles.StyledMessage;
import org.gridprovenance.client.relationships.RelationObject;
import org.gridprovenance.client.relationships.RelationSubject;
import org.gridprovenance.client.exception.CSLException;

/**
 * InteractionRecord Interface<p>
 * CSL of EU Provenance 
 *
 * @author Sheng Jiang
 */

public interface InteractionRecord {

	/**
	 * Add a InteractionPAssertion using Verbatim Documentation Style 
	 * @param message As Element
	 */
	public GlobalPAssertionKey addInteractionPAssertion (Element message) throws Exception, CSLException;

	/**
	 * Add a InteractionPAssertion
	 * @param message As Element
	 * @param ds documentationStyle As java.net.URI
	 */
	public GlobalPAssertionKey addInteractionPAssertion (Element message, URI ds) throws Exception, CSLException;
   
	/**
	 * Add an ActorStatePAssertion using Verbatim Documentation Style
	 * @param actorState As Element
	 */
	public GlobalPAssertionKey addActorStatePAssertion (Element actorState) throws Exception, CSLException;

	/**
	 * Add an ActorStatePAssertion
	 * @param actorState As Element
	 * @param ds documentationStyle as java.net.URI
	 */
	public GlobalPAssertionKey addActorStatePAssertion (Element actorState, URI ds) throws Exception, CSLException;

	/**
	 * Add a relationshipPAssertion
	 */
	public GlobalPAssertionKey addRelationshipPAssertion (RelationObject [] relationObject, URI relationship, RelationSubject relationSubject) throws Exception, CSLException;
	
	/**
	 * Get the InteractionKey
	 * @return InteractionKey
	 */
	public InteractionKey getInteractionKey () throws CSLException;
	
	/**
	 * Get the Record Address
	 * @return recordAddress As EndpointReference
	 */
	public EndpointReference getRecordPortAddress () throws CSLException;

	/**
	 * get the Side <p>
	 * return true if it is sender side
	 * return false if it is receive side
	 */
	public ViewKind getViewKind () throws CSLException;
	
	/**
	 * get the Asserter
	 * @return asserter As Principal
	 */
	public Principal getAsserter () throws CSLException;

	/**
	 * set the InteractionKey
	 * @param ik As InteractionKey
	 */
	public void setInteractionKey (InteractionKey ik) throws CSLException;
	
	/**
	 * Set the Side <p>
	 * true if it is sender side
	 * false if it is receive side
	 */
	public void setViewKind (ViewKind viewKind) throws CSLException;
	
	/**
	 * set the Asserter
	 * @param asserter As Principal
	 */
	public void setAsserter (Principal asserter) throws CSLException;

	/**
	 * Record PAssertions to Provenance Service
	 */
	public RecordResult record () throws Exception, CSLException;
}