/* 
 * <START_COPYRIGHT>
 * Copyright (c) 2005 University of Southampton
 * See CPL.txt for license information.
 * http://www.opensource.org/license/cpl1.0.txt
 * <END_COPYRIGHT>
 */

package org.gridprovenance.client;

import org.apache.axis.message.addressing.EndpointReference;
import java.security.Principal;
import java.util.Map;
import org.gridprovenance.client.exception.CSLException;

/**
 * ProvenanceService Interface<p>
 * CSL of EU Provenance 
 *
 * @author Sheng Jiang
 */

public interface ProvenanceService {

	/**
	 * Get Record Port
	 * @return recordPort As EndpointReference
	 */
	public EndpointReference getRecordPortAddress () throws CSLException;

	/**
	 * Get XQuery Port
	 * @return xQueryPort As EndpointReference
	 */
	public EndpointReference getXQueryPortAddress () throws CSLException;

	/**
	 * Get PQuery Port
	 * @return pQueryPort As EndpointReference
	 */
	public EndpointReference getPQueryPortAddress () throws CSLException;

	/**
	 * Create a new InteractionRecord
	 * @return InteractionRecord
	 */
	public InteractionRecord newInteractionRecord (EndpointReference msgSink, EndpointReference msgSource, Principal asserter, ViewKind vk) throws Exception, CSLException;
	
	/**
	 * Create a new XQuery
	 * @return XQuery
	 */
	public XQuery newXQuery (String optionString) throws CSLException;
	
	/**
	 * Create a new PQuery
	 * @return PQuery
	 */
	public PQuery newPQuery (String xPath1, String xPath2, Map namespaces) throws CSLException;
	
	/**
	 * Create a new PQuery
	 * @return PQuery
	 */
	public PQuery newPQuery (String xPath1, Map namespaces1, String xPath2, Map namespaces2) throws CSLException;
}
