/* 
 * <START_COPYRIGHT>
 * Copyright (c) 2005 University of Southampton
 * See CPL.txt for license information.
 * http://www.opensource.org/license/cpl1.0.txt
 * <END_COPYRIGHT>
 */

package org.gridprovenance.client.impl;


import java.net.URI;
import java.util.ArrayList;
import java.security.Principal;
import org.w3c.dom.Element;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.EndpointReference;
import org.gridprovenance.client.InteractionRecord;
import org.gridprovenance.client.RecordResult;
import org.gridprovenance.client.InteractionKey;
import org.gridprovenance.client.ViewKind;
import org.gridprovenance.client.GlobalPAssertionKey;
import org.gridprovenance.client.documentationstyles.StyledMessage;
import org.gridprovenance.client.generated.ProvenanceServiceAddressingLocator;
import org.gridprovenance.client.generated.pstruct.InteractionPAssertion;
import org.gridprovenance.client.generated.pstruct.ActorStatePAssertion;
import org.gridprovenance.client.generated.pstruct.RelationshipPAssertion;
import org.gridprovenance.client.generated.pstruct.RelationshipPAssertionSubjectId;
import org.gridprovenance.client.generated.pstruct.ObjectId;
import org.gridprovenance.client.generated.pstruct.Link;
import org.gridprovenance.client.generated.pstruct.DataAccessor;
import org.gridprovenance.client.generated.pstruct.LocalPAssertionId;
import org.gridprovenance.client.generated.record.RecordAck;
import org.gridprovenance.client.generated.record.IdentifiedContent;
import org.gridprovenance.client.generated.record.Content;
import org.gridprovenance.client.generated.record.RecordPortType;
import org.gridprovenance.client.relationships.RelationObject;
import org.gridprovenance.client.relationships.RelationSubject;
import org.gridprovenance.client.utilities.AsserterUtil;
import org.gridprovenance.client.utilities.InteractionIdFactory;
import org.gridprovenance.client.utilities.LocalPAssertionIdFactory;
import org.gridprovenance.client.utilities.InteractionKeyUtil;
import org.gridprovenance.client.utilities.AddressUtil;
import org.gridprovenance.client.utilities.URIUtil;
import org.gridprovenance.client.utilities.DataAccessorUtil;
import org.gridprovenance.client.exception.CSLException;

/**
 * An implement of Record interface<p>
 * CSL of EU Provenance
 *
 * @author Sheng Jiang
 */

public class InteractionRecordImpl implements InteractionRecord {
	private EndpointReference recordPortAddress;
	private ArrayList pAssertions = new ArrayList();
	private InteractionKey interactionKey;
	private ViewKind vk;
	private Principal asserter;

   public InteractionRecordImpl() {
   }
   
   public InteractionRecordImpl (InteractionRecord inputIR)
   throws CSLException {
	   this.recordPortAddress = inputIR.getRecordPortAddress();
	   this.vk = inputIR.getViewKind();
	   this.interactionKey = inputIR.getInteractionKey();
	   this.asserter = inputIR.getAsserter();
   }

   public InteractionRecordImpl (ViewKind vk, InteractionKey ik, Principal asserter, EndpointReference recordPortAddress){
	   this.recordPortAddress = recordPortAddress;
	   this.vk = vk;
	   this.interactionKey = ik;
	   this.asserter = asserter;
   }
   
   public InteractionRecordImpl (ViewKind vk, EndpointReference sink, EndpointReference source, Principal asserter, EndpointReference recordPortAddress)
   throws Exception, CSLException {
	   this.recordPortAddress = recordPortAddress;
	   this.vk = vk;
	   this.interactionKey = new InteractionKeyImpl (InteractionIdFactory.newInstance(), sink, source);
	   this.asserter = asserter;
   }

   public GlobalPAssertionKey addInteractionPAssertion (Element message)
   throws Exception, CSLException {
	   URI defaultDS = new URI("http://www.gridprovenance.org/schemas/ontologies/DocumentationStyle.Verbatim");
	   MessageElement [] messageArray = new MessageElement [1];
	   messageArray [0] = new MessageElement(message);
	   org.gridprovenance.client.generated.pstruct.Content messageContent = new org.gridprovenance.client.generated.pstruct.Content(messageArray);
	   org.apache.axis.types.URI axisDS = URIUtil.toAxisURI (defaultDS);
	   LocalPAssertionId lpid = LocalPAssertionIdFactory.newInstance();
	   InteractionPAssertion ipa = new InteractionPAssertion(messageContent, axisDS, lpid);
	   this.pAssertions.add(new Content (null, ipa, null, null));
	   GlobalPAssertionKey gpkg = new GlobalPAssertionKeyImpl(this.interactionKey, this.vk, lpid.toString());
	   return (gpkg);
   }

   public GlobalPAssertionKey addInteractionPAssertion (Element message, URI ds)
   throws Exception, CSLException {
	   MessageElement [] messageArray = new MessageElement [1];
	   messageArray [0] = new MessageElement(message);
	   org.gridprovenance.client.generated.pstruct.Content messageContent = new org.gridprovenance.client.generated.pstruct.Content(messageArray);
	   org.apache.axis.types.URI axisDS = URIUtil.toAxisURI (ds);
	   //ds here should be used to get actorState into Styled Message
	   LocalPAssertionId lpid = LocalPAssertionIdFactory.newInstance();
	   InteractionPAssertion ipa = new InteractionPAssertion(messageContent, axisDS, lpid);
	   this.pAssertions.add(new Content (null, ipa, null, null));
	   GlobalPAssertionKey gpkg = new GlobalPAssertionKeyImpl(this.interactionKey, this.vk, lpid.toString());
	   return (gpkg);
   }
   
   public GlobalPAssertionKey addActorStatePAssertion (Element actorState)
   throws Exception, CSLException {
	   URI defaultDS = new URI("http://www.gridprovenance.org/schemas/ontologies/DocumentationStyle.Verbatim");
	   MessageElement [] actorStateArray = new MessageElement [1];
	   actorStateArray [0] = new MessageElement(actorState);
	   org.gridprovenance.client.generated.pstruct.Content actorStateContent = new org.gridprovenance.client.generated.pstruct.Content(actorStateArray);
	   org.apache.axis.types.URI axisDS = URIUtil.toAxisURI (defaultDS);
	   LocalPAssertionId lpid = LocalPAssertionIdFactory.newInstance();
	   ActorStatePAssertion aspa = new ActorStatePAssertion(actorStateContent, axisDS, lpid);
	   this.pAssertions.add(new Content (aspa, null, null, null));
	   GlobalPAssertionKey gpkg = new GlobalPAssertionKeyImpl(this.interactionKey, this.vk, lpid.toString());
	   return (gpkg);
   }

   public GlobalPAssertionKey addActorStatePAssertion (Element actorState, URI ds)
   throws Exception, CSLException {
	   MessageElement [] actorStateArray = new MessageElement [1];
	   actorStateArray [0] = new MessageElement(actorState);
	   org.gridprovenance.client.generated.pstruct.Content actorStateContent = new org.gridprovenance.client.generated.pstruct.Content(actorStateArray);
	   org.apache.axis.types.URI axisDS = URIUtil.toAxisURI (ds);
	   //ds here should be used to get actorState into Styled Message
	   LocalPAssertionId lpid = LocalPAssertionIdFactory.newInstance();
	   ActorStatePAssertion aspa = new ActorStatePAssertion(actorStateContent, axisDS, lpid);
	   this.pAssertions.add(new Content (aspa, null, null, null));
	   GlobalPAssertionKey gpkg = new GlobalPAssertionKeyImpl(this.interactionKey, this.vk, lpid.toString());
	   return (gpkg);
   }

   public GlobalPAssertionKey addRelationshipPAssertion (RelationObject [] relationObject, URI relationship, RelationSubject relationSubject)
   throws Exception, CSLException {
	   ObjectId [] objectId = new ObjectId [relationObject.length];
	   for (int i = 0; i < relationObject.length; i++) {
		   Link olink = new Link(AddressUtil.newAddress(relationObject[i].getObjectLink()));
		   org.apache.axis.types.URI opn = URIUtil.toAxisURI(relationObject[i].getParameterName());
		   objectId [i] = new ObjectId (olink, opn);
		   objectId [i].setInteractionKey(InteractionKeyUtil.toPStructIK(relationObject[i].getGPAK().getInteractionKey()));
		   LocalPAssertionId olpaid = new LocalPAssertionId (relationObject[i].getGPAK().getLocalPAssertionId());
		   objectId [i].setLocalPAssertionId(olpaid);
		   objectId [i].setViewKind(org.gridprovenance.client.generated.pstruct.ViewKind.fromString(relationObject[i].getGPAK().getViewKind().getViewKind()));
		   if (relationObject[i].getDataAccessor()!= null) {
			   DataAccessor oda = DataAccessorUtil.toPStructIK(relationObject[i].getDataAccessor());
			   objectId [i].setDataAccessor(oda);
		   }
	   }
	   
	   DataAccessor sda = new DataAccessor ();
	   if (relationSubject.getDataAccessor() != null) {
		    sda = DataAccessorUtil.toPStructIK(relationSubject.getDataAccessor());
	   }
	   
	   LocalPAssertionId slpaid = new LocalPAssertionId (relationSubject.getGPAK().getLocalPAssertionId());
	   org.apache.axis.types.URI spn = URIUtil.toAxisURI(relationSubject.getParameterName());
	   RelationshipPAssertionSubjectId subjectId = new RelationshipPAssertionSubjectId(sda, slpaid, spn);
	   LocalPAssertionId lpid = LocalPAssertionIdFactory.newInstance();
	   RelationshipPAssertion rpa = new RelationshipPAssertion(lpid, objectId, URIUtil.toAxisURI(relationship), subjectId);
	   this.pAssertions.add(new Content (null, null, rpa, null));
	   GlobalPAssertionKey gpkg = new GlobalPAssertionKeyImpl(this.interactionKey, this.vk, lpid.toString());
	   return (gpkg);
   }
   
   public InteractionKey getInteractionKey ()
   throws CSLException {
	   return (this.interactionKey);
   }
   
   public EndpointReference getRecordPortAddress ()
   throws CSLException {
	   return (this.recordPortAddress);
   }
   
	public ViewKind getViewKind ()
	throws CSLException {
		return this.vk;
	}
	
	public Principal getAsserter ()
	throws CSLException {
		return (this.asserter);
	}

	public void setInteractionKey (InteractionKey ik)
	throws CSLException {
		this.interactionKey = ik;
	}
	
	public void setViewKind (ViewKind vk)
	throws CSLException {
		this.vk = vk;
	}
	
	public void setAsserter (Principal asserter)
	throws CSLException {
		this.asserter = asserter;
	}

	public RecordResult record ()
	throws Exception, CSLException {
		ProvenanceServiceAddressingLocator rlocator = new ProvenanceServiceAddressingLocator();
		RecordPortType rpt = rlocator.getRecordPort (this.recordPortAddress);

		Content [] contentArray = (Content[]) this.pAssertions.toArray(new Content [0]);
		IdentifiedContent [] iCArray = new IdentifiedContent [1];
		org.gridprovenance.client.generated.pstruct.InteractionKey ik = InteractionKeyUtil.toPStructIK(this.interactionKey);
		org.gridprovenance.client.generated.pstruct.ViewKind viewk = org.gridprovenance.client.generated.pstruct.ViewKind.fromString(this.vk.getViewKind());
		
		iCArray [0] = new IdentifiedContent (AsserterUtil.newAsserter (this.asserter), contentArray, ik, viewk);
		org.gridprovenance.client.generated.record.Record record = new org.gridprovenance.client.generated.record.Record (iCArray);

		RecordResult rr = new RecordResultImpl();
		try{
			RecordAck rAck = rpt.record (record);
			rr = new RecordResultImpl(rAck.getERROR());
		} catch (Exception e) {
			e.printStackTrace();
		}

		return (rr);
	}
}