/*
 * <START_COPYRIGHT>
 * Copyright (c) 2006 University of Southampton
 * See CPL.txt for license information.
 * http://www.opensource.org/license/cpl1.0.txt
 * <END_COPYRIGHT>
 */

package org.gridprovenance.client.test;

import org.w3c.dom.Element;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.security.Principal;
import javax.security.auth.x500.X500Principal;
import java.net.URI;
import org.apache.axis.message.addressing.EndpointReference;

import org.gridprovenance.client.ProvenanceService;
import org.gridprovenance.client.InteractionRecord;
import org.gridprovenance.client.RecordResult;
import org.gridprovenance.client.XQuery;
import org.gridprovenance.client.XQueryResult;
import org.gridprovenance.client.GlobalPAssertionKey;
import org.gridprovenance.client.ViewKind;
import org.gridprovenance.client.RecordPort;
import org.gridprovenance.client.XQueryPort;
import org.gridprovenance.client.PQueryPort;

import org.gridprovenance.client.impl.ProvenanceServiceImpl;
import org.gridprovenance.client.impl.ViewKindImpl;
import org.gridprovenance.client.impl.RecordPortImpl;
import org.gridprovenance.client.impl.XQueryPortImpl;
import org.gridprovenance.client.impl.PQueryPortImpl;

import org.gridprovenance.client.relationships.DataAccessor;
import org.gridprovenance.client.relationships.RelationSubject;
import org.gridprovenance.client.relationships.RelationObject;
import org.gridprovenance.client.relationships.XPathDataAccessor;

import org.gridprovenance.client.utilities.ElementUtil;

import org.gridprovenance.client.exception.CSLException;

/**
 * Demonstrates how to use the Client Side Library (CSL) record<p>
 * interaction p-assertions, actor state p-assertions, relationship<p>
 * p-assertions and query the recorded p-assertions back.<p>
 * CSL of EU Provenance
 *
 * @author Sheng Jiang, Paul Groth
 */

public class ApplicationAPIExample
{
    public static void main(String[] args) throws Exception, CSLException {
        if (args.length != 1) {
            System.out.println (" Usage:\tjava org.gridprovenance.test.ApplicationAPIExample <provenance store url>");
            System.out.println ("\t\texample url: http://localhost:8080/preserv");
            System.out.println ("\t\tThere should be NO trailing / behind preserv");
            System.exit(0);
        }

        System.out.println("ApplicationAPIExample");
        System.out.println("The scenario():\tWe simulate a sender sending a message to a receiver.");
        System.out.println("\t\tThe sender uses part of its state in the the message so");
        System.out.println("\t\tit makes a relationship p-assertion between its actor");
        System.out.println("\t\tstate p-assertion and its interaction p-assertion.\n");
        System.out.println("Press any key to continue...");
        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
        br.readLine();

        /**
         * The following parameters should be set up / provide by applications,
         * either from the configure files or generated during application executing.
         * Here, as a simple example, we use the pre-set values instead.
         */

        RecordPort recordPort = new RecordPortImpl (args[0] + "/record");
        XQueryPort xQueryPort = new XQueryPortImpl (args[0] + "/xquery");
        PQueryPort pQueryPort = new PQueryPortImpl (); //PQuery function is not aviable yet.
        //Provenance service represented a service which has three porttypes.
        //Even the empty port should be initialled.
        ViewKind vk = ViewKindImpl.isSender;
        EndpointReference source = new EndpointReference("http://www.sender.com/sender");
        EndpointReference sink   = new EndpointReference("http://www.receiver.com/receiver");
        Principal asserter = new X500Principal("CN=Sheng, OU=ecs, O=soton, C=UK");
        URI documentationStyle = new URI("http://www.gridprovenance.org/schemas/ontologies/DocumentationStyle.Verbatim");
        Element actorStates = ElementUtil.newElement("<cpu>1.2 GHz</cpu>");
        Element message = ElementUtil.newElement("<hello>Sheng</hello>");

        //ElementUtil is an internal utility to convert String into Element
        //Applications should not use ElementUtil. They should make Element and Element array themselves

        //Other example message, which has converted from a soap message
        /*      Element message = ElementUtil.newElement("" +
                "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope\">" +
                "<soapenv:Header/>" +
                "<soapenv:Body>" +
                "<agreeResponse/>" +
                "</soapenv:Body>" +
                "</soapenv:Envelope>"); */

        //If needed, the original message can be transformed by CLS utilities StyleFactory
        //However, this function is not supported in this release yet.
        //StyledMessage styledMessage = StyleFactory.getStyler(documentationStyle).style(message);

        /**
         * Create a ProvenanceService for recording and xquerying.
         */

        ProvenanceService ps = new ProvenanceServiceImpl (recordPort, xQueryPort, pQueryPort);

        /**
         * In order to create a new record object, message source, message sink, asserter
         * and side (sender or receiver) should be given.
         */

        InteractionRecord interactionRecordInstance = ps.newInteractionRecord (sink, source, asserter, vk);

        System.out.println("Building three kinds of p-assertions within the InteractionRecord...");

		GlobalPAssertionKey gpakIPA = interactionRecordInstance.addInteractionPAssertion(message, documentationStyle);
		//if no documentationStyle given, Verbatim will be applied
		
		GlobalPAssertionKey gpakAPA = interactionRecordInstance.addActorStatePAssertion (actorStates, documentationStyle);
		//if no documentationStyle given, Verbatim will be applied

        /*
         * We create a "usage" relationship between the above 2 PAssertions. This
         * relationship says message represented by InteractionPAssertion used the
         * actor state represented by ActorStatePAssertion. In order to do so, there
         * are a few more parameters that should be given by applications. Applications
         * may use relation helper to get or set right relationships.
         *
         * In order to create the relationship PAssertion, the following parameters
         * should be set up / provide by applications.
         */

        DataAccessor subjectDA = new XPathDataAccessor("/soap:envelope/soap:body/message:hello"); //DataAccssor are optional
        URI subjectPAParameterName = new URI("http://www.pasoa.org/schemas/ontologies/relationships/usage.user");
        DataAccessor objectDA = new XPathDataAccessor("/server:cpu"); //DataAccssor are optional
        EndpointReference objectLink = new EndpointReference ("http://www.pasoa.org/xquery");
        //where objectPA can be queried back
        URI objectPAParameterName = new URI("http://www.pasoa.org/schemas/ontologies/relationships/usage.usedItem");
        URI relationship = new URI ("http://www.pasoa.org/schemas/ontologies/relationships/use");

        RelationObject [] relationObject = new RelationObject [1]; //only because we know there are one object
        relationObject [0] = new RelationObject(objectDA, objectPAParameterName, gpakAPA, objectLink);
        RelationSubject relationSubject = new RelationSubject(subjectDA, subjectPAParameterName, gpakIPA);

        interactionRecordInstance.addRelationshipPAssertion (relationObject, relationship, relationSubject);

        System.out.println("Recording the p-assertions in provenance service " + recordPort.getRecordPort().toString());
        RecordResult rResult = interactionRecordInstance.record();
        rResult.printOut();

        System.out.println ("Three p-assertions recorded");
        System.out.println ("Press any key to continue...");
        br.readLine();
        System.out.println ("***************************************");
        System.out.println ("Querying the content of the provenance store");
        System.out.println ("The entire content of the store will be displayed");
        System.out.println ("Press any key to continue...");
        br.readLine();

        /**
         * Create a XQuery Instance
         */

        String xQueryString = "<result>{for $n in $ps:pstruct return $n}</result>";

        XQuery xQueryInstance = ps.newXQuery(xQueryString);

        XQueryResult xqResult = xQueryInstance.execute();
        xqResult.printOut();

        System.out.println ("\nDone\n");
        System.out.println ("Thanks for using ApplicationAPIExample of this Client Side Library!");
        System.out.println ("Have fun creating your own provenance aware application!\n");
    }
}