/* 
 * <START_COPYRIGHT>
 * Copyright (c) 2006 University of Southampton
 * See CPL.txt for license information.
 * http://www.opensource.org/license/cpl1.0.txt
 * <END_COPYRIGHT>
 */

package org.gridprovenance.client.test;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

import org.apache.axis.types.URI;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.gridprovenance.client.generated.ProvenanceServiceAddressingLocator;
import org.gridprovenance.client.generated.record.RecordPortType;
import org.gridprovenance.client.generated.record.IdentifiedContent;
import org.gridprovenance.client.generated.record.Record;
import org.gridprovenance.client.generated.pstruct.ActorStatePAssertion;
import org.gridprovenance.client.generated.pstruct.Address;
import org.gridprovenance.client.generated.pstruct.Asserter;
import org.gridprovenance.client.generated.pstruct.InteractionKey;
import org.gridprovenance.client.generated.pstruct.InteractionPAssertion;
import org.gridprovenance.client.generated.pstruct.Link;
import org.gridprovenance.client.generated.pstruct.LocalPAssertionId;
import org.gridprovenance.client.generated.pstruct.ObjectId;
import org.gridprovenance.client.generated.pstruct.RelationshipPAssertion;
import org.gridprovenance.client.generated.pstruct.RelationshipPAssertionSubjectId;
import org.gridprovenance.client.generated.pstruct.ViewKind;
import org.gridprovenance.client.generated.xquery.XQueryPortType;
import org.gridprovenance.client.generated.xquery.Query;
import org.gridprovenance.client.generated.xquery.QueryAck;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;

/**
 * Demonstrates how to use the Server API of the Client Side Library (CSL)<p>
 * to record interaction p-assertions, actor state p-assertions,<p>
 * relationship p-assertions and query the recorded p-assertions back.<p>
 * CSL of EU Provenance
 * 
 * @author Sheng Jiang, Paul Groth
 */

public class ServerAPIExample
{
    public static void main(String[] args) throws Exception
    {
        if (args.length != 1) {
            System.out.println (" Usage:\tjava org.gridprovenance.test.ServerAPIExample <provenance store url>");
            System.out.println ("\t\texample url: http://localhost:8080/preserv");
            System.out.println ("\t\tThere should be NO trailing / behind preserv");
            System.out.println("! Warning:\tServer API is primarily intended to be use by application");
            System.out.println("\t\tdevelopers! Unless you really want to learn the details of");
            System.out.println("\t\tServer API, please use ClientLibExample instead.\n");
            System.exit(0);
        }
        
        // Here we create the urls for recording and querying the provenance store
        // send record messages to the record port (provenanceStore)
        // send query messages to the xquery port (proveanceStoreQuery)
        String provenanceStore = args[0] + "/record";
        String provenanceStoreQuery = args[0] + "/xquery";
        
        ServerAPIExample cle = new ServerAPIExample();
        
        System.out.println("ServerAPIExample\n");
        System.out.println("! Warning:\tServer API is primarily intended to be use by application");
        System.out.println("\t\tdevelopers! Unless you really want to learn the details of");
        System.out.println("\t\tServer API, please use ClientLibExample instead.\n");
        System.out.println("The scenario:\tWe simulate a sender sending a message to a receiver.");
        System.out.println("\t\tThe sender uses part of its state in the the message so");
        System.out.println("\t\tit makes a relationship p-assertion between its actor");
        System.out.println("\t\tstate p-assertion and its interaction p-assertion.\n");
        System.out.println("Press any key to continue...");
        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
        br.readLine();

        System.out.println("We now create the interaction key");

        // Create source address and covert it into a MessageElement array
        String source = "<URL>http://www.sender.com/sender</URL>";
        DocumentBuilderFactory dbf1 = DocumentBuilderFactory.newInstance();
        DocumentBuilder db1 = dbf1.newDocumentBuilder();
        Document msgDoc1 = db1.parse(new InputSource(new StringReader(source)));
        Element sourceBody = msgDoc1.getDocumentElement();
        MessageElement[] sourceArray = new MessageElement[1];
        sourceArray[0] = new MessageElement(sourceBody);

        // Create sink address and convert it into a MessageElement array
        String sink = "<URL>http://www.receiver.com/receiver</URL>";
        DocumentBuilderFactory dbf2 = DocumentBuilderFactory.newInstance();
        DocumentBuilder db2 = dbf2.newDocumentBuilder();
        Document msgDoc2 = db2.parse(new InputSource(new StringReader(sink)));
        Element sinkBody = msgDoc2.getDocumentElement();
        MessageElement[] sinkArray = new MessageElement[1];
        sinkArray[0] = new MessageElement(sinkBody);
        
        // Create an interactionId, this should be unique!(as URI)
        URI interactionId = new URI ("http://uniqueId/" + System.currentTimeMillis());

        InteractionKey ik = new InteractionKey (interactionId, new Address(sinkArray), new Address(sourceArray));
        
        System.out.println ("The interaction key has been created");
        System.out.println ();
        
        System.out.println ("We are the sender/client view of the interaction");
        ViewKind vk = ViewKind.isSender;
        System.out.println ();
        
        // Create a asserter
        String as = "<url>http://www.users.com/SomeGuy</url>";
        DocumentBuilderFactory dbf3 = DocumentBuilderFactory.newInstance();
        DocumentBuilder db3 = dbf3.newDocumentBuilder();
        Document msgDoc3 = db3.parse(new InputSource(new StringReader(as)));
        Element asBody = msgDoc3.getDocumentElement();
        MessageElement[] asArray = new MessageElement[1];
        asArray[0] = new MessageElement(asBody);
        Asserter asserter = new Asserter(asArray);
        System.out.println ("The asserter is identified by the following url: http://www.users.com/SomeGuy");
        System.out.println ("Press any key to continue...");
        br.readLine();
        
        
        System.out.println("Building three kind of p-assertions...");
        
        InteractionPAssertion ipa = cle.createInteractionPAssertion();
        ActorStatePAssertion  asa = cle.createActorStatePAssertion();
        RelationshipPAssertion rel = cle.createRelationshipPAssertion (ik, vk, provenanceStore); 
        
        System.out.println("Three p-assertions have been created");
        System.out.println("Creating Record objects for each p-assertion");
        
        org.gridprovenance.client.generated.record.Content[] content1 = new org.gridprovenance.client.generated.record.Content[1];
        content1[0] = new org.gridprovenance.client.generated.record.Content (null, ipa, null, null);

        IdentifiedContent[] ICipa = new IdentifiedContent[1];
        ICipa[0] = new IdentifiedContent(asserter, content1, ik, vk);

        org.gridprovenance.client.generated.record.Content[] content2 = new org.gridprovenance.client.generated.record.Content[1];
        content2[0] = new org.gridprovenance.client.generated.record.Content (asa, null, null, null);

        IdentifiedContent[] ICasa = new IdentifiedContent[1];
        ICasa[0] = new IdentifiedContent(asserter, content2, ik, vk);

        org.gridprovenance.client.generated.record.Content[] content3 = new org.gridprovenance.client.generated.record.Content[1];
        content3[0] = new org.gridprovenance.client.generated.record.Content (null, null, rel, null);

        IdentifiedContent[] ICrel = new IdentifiedContent[1];
        ICrel[0] = new IdentifiedContent(asserter, content3, ik, vk);

        Record recIpa = new Record (ICipa);
        Record recAsa = new Record (ICasa); 
        Record recRel = new Record (ICrel);
        
        System.out.println("Recording the p-assertions in provenance store " + provenanceStore);
        
        ProvenanceServiceAddressingLocator locator = new ProvenanceServiceAddressingLocator ();
        EndpointReferenceType endpoint = new EndpointReferenceType();
        endpoint.setAddress(new org.apache.axis.message.addressing.Address(provenanceStore));

        RecordPortType rpt = locator.getRecordPort(endpoint);
        try {
        	rpt.record(recIpa);
        	rpt.record(recAsa);
        	rpt.record(recRel);
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        System.out.println ("Three p-assertions recorded");
        System.out.println ("Press any key to continue...");
        br.readLine();
        System.out.println ("***************************************");
        System.out.println ("Querying the content of the provenance store");
        System.out.println ("    The entire content of the store will be displayed");
        System.out.println ("Press any key to continue...");
        br.readLine();

        //query the recorded PAssertions
        Query query = new Query("<result>{for $n in $ps:pstruct return $n}</result>");

        ProvenanceServiceAddressingLocator qlocator = new ProvenanceServiceAddressingLocator ();
        EndpointReferenceType qendpoint = new EndpointReferenceType();
        qendpoint.setAddress(new org.apache.axis.message.addressing.Address(provenanceStoreQuery));

        System.out.println("Querying the p-assertions in provenance store " + provenanceStoreQuery);
        
        XQueryPortType qpt = qlocator.getXQueryPort(qendpoint);
        try {
                QueryAck qAck = qpt.query(query);
		MessageElement[] resultArray = qAck.get_any();
	        System.out.println (resultArray[0].toString());
        } catch (Exception e) {
            e.printStackTrace();
        }

        System.out.println ();
        System.out.println ("Done");
        System.out.println ();
        System.out.println ();
        System.out.println ("Thanks for using ServerAPIExample!");
        System.out.println ("Have fun creating your own provenance aware application!");
    }
    
    private InteractionPAssertion createInteractionPAssertion() 
        throws Exception
    {
        // Create an interaction p-assertion
        // First we make a local p-assertion id and then
        // we make a documentationStyle. In this case we 
        // call it verbatium.
        //
        // In most cases, you'll be grabing the messageBody from the message
        // being sent between parties. So a SOAP message, or a CORBA message.
        // With this example we'll just use a hard coded message body.
        
        LocalPAssertionId localPAssertionId = new LocalPAssertionId (1);
        URI documentationStyle = new URI("http://www.pasoa.org/schemas/ontologies/DocumentationStyle.Verbatim");
        
        // The contents of a message
        String message = "<hello>Blah 1</hello>";
        
        // Convert it into a  MessageElement array
        DocumentBuilderFactory dbf4 = DocumentBuilderFactory.newInstance();
        DocumentBuilder db4 = dbf4.newDocumentBuilder();
        Document msgDoc4 = db4.parse(new InputSource(new StringReader(message)));
        Element messageBody = msgDoc4.getDocumentElement();
        MessageElement[] messageArray = new MessageElement[1];
        messageArray[0] = new MessageElement(messageBody);
        org.gridprovenance.client.generated.pstruct.Content messageContent = new org.gridprovenance.client.generated.pstruct.Content(messageArray);
        
        InteractionPAssertion ipa = new InteractionPAssertion(messageContent, documentationStyle, localPAssertionId);
        
        return ipa;
    }
    
    private ActorStatePAssertion createActorStatePAssertion() 
        throws Exception
    {
        // Create an actor state p-assertion
        // Just like the interaction p-assertion this p-assertion
        // needs a local p-assertion id. Remember, all the p-assertions
        // in one view need a different id. Therefore, we give this assertion
        // the id of "2" instead of "1".
        //
        // Again you'll typically be getting some state from the actor,
        // translating it to XML to create the actor state p-assertion
        // In this example, we just use a hard coded string.
        
        LocalPAssertionId localPAssertionId = new LocalPAssertionId (2);
        
        // The contents of the assertion
        String actorStateBody = "<state>Blah 2</state>";
        
        // Convert it into a MessageElement Array
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = dbf.newDocumentBuilder();
        Document msgDoc = db.parse(new InputSource(new StringReader(actorStateBody)));
        Element stateBody = msgDoc.getDocumentElement();
        MessageElement[] stateArray = new MessageElement[1];
        stateArray[0] = new MessageElement(stateBody);
        org.gridprovenance.client.generated.pstruct.Content stateContent = new org.gridprovenance.client.generated.pstruct.Content(stateArray);

        ActorStatePAssertion asa = new ActorStatePAssertion(stateContent, null, localPAssertionId);
        return asa;
    }
    
  private RelationshipPAssertion createRelationshipPAssertion (InteractionKey ik, ViewKind view, String provenanceStore)
         throws Exception
    {
        // Create a relationship p-assertion
        // Again a different local p-assertion id
        //
        // We'll create a "usage" relationship between the interaction p-assertion
        // and the actor state p-assertion. This relationship says that 
        // message represented by interaction p-assertion "1" used the actor state
        // represented by actor state p-assertion "2".
        // There are no data accessors or links so we pass null.
     
        // Create the information to identify the subject of the relationship
        // Remember, parameter names must be identified and they need to be URIs
        LocalPAssertionId localPAssertionId = new LocalPAssertionId (3);

        LocalPAssertionId subjectLocalPAssertionId = new LocalPAssertionId(1);
        URI subjectParameterName = new URI("http://www.pasoa.org/schemas/ontologies/relationships/usage.user");
        RelationshipPAssertionSubjectId subjectId = new RelationshipPAssertionSubjectId(null, subjectLocalPAssertionId, subjectParameterName);
        
        // Create the information to identify the object of the relationship
        
        // We get the interaction key and view where the actor state p-assertion 
        // is from the parameters to this method. In this case, the actor state p-assertion
        // is in the same interaction record and view as the interaction p-assertion. 
        // In many other cases, the object of the relation could be in another interaction record
        // and in another view.

        LocalPAssertionId objectLocalPAssertionId = new LocalPAssertionId (2); // points to the actor state p-assertion

        // Convert String provenanceStore to MessageElement
        DocumentBuilderFactory dbf5 = DocumentBuilderFactory.newInstance();
        DocumentBuilder db5 = dbf5.newDocumentBuilder();
        Document msgDoc5 = db5.parse(new InputSource(new StringReader("<URL>"+provenanceStore+"</URL>")));
        Element storeRefBody = msgDoc5.getDocumentElement();
        MessageElement[] storeRefArray = new MessageElement[1];
        storeRefArray[0] = new MessageElement(storeRefBody);
        Address storeRef = new Address (storeRefArray);
	
        Link objectLink = new Link(storeRef);
        URI objectParameterName = new URI("http://www.pasoa.org/schemas/ontologies/relationships/usage.usedItem");

        ObjectId[] objectId = new ObjectId[1];
        objectId[0] = new ObjectId (objectLink, objectParameterName);
        objectId[0].setInteractionKey(ik);
        objectId[0].setLocalPAssertionId(objectLocalPAssertionId);
        objectId[0].setViewKind(view);
        
        // Create the "use" relation as a URI
        URI relation = new URI ("http://www.pasoa.org/schemas/ontologies/relationships/use");

        // Finally, create the relationship object and return it.
        RelationshipPAssertion rel = new RelationshipPAssertion(localPAssertionId, objectId, relation, 
                                            subjectId);
        
        return rel;
    }
}
